@@ -30,5 +30,6 @@ ENCMAP = {
 }
 
 
-add_map("navajo", ENCMAP, ignore_case="both", sep=" ", pattern=r"^navajo$", printables_rate=1.)
+add_map("navajo", ENCMAP, ignore_case="both", sep=" ", pattern=r"^navajo$", printables_rate=1.,
+        expansion_factor=(6.2, .8))
 
