@@ -24,6 +24,6 @@ ENCMAP = {
 }
 
 
-add_map("radio", ENCMAP, sep=" ", ignore_case="both",
-        pattern=r"^(?:military|nato|radio)(?:(?:[-_]phonetic)?(?:[-_]alphabet)?)?$", printables_rate=1.)
+add_map("radio", ENCMAP, sep=" ", ignore_case="both", printables_rate=1., expansion_factor=(5.5, .3),
+        pattern=r"^(?:military|nato|radio)(?:(?:[-_]phonetic)?(?:[-_]alphabet)?)?$")
 
