@@ -30,5 +30,6 @@ ENCMAP = {
 }
 
 
-add_map("tom-tom", ENCMAP, ".", " ", ignore_case="both", pattern=r"^tom-?tom([-_]?.{3})?$", printables_rate=1.)
+add_map("tom-tom", ENCMAP, ".", " ", ignore_case="both", pattern=r"^tom-?tom([-_]?.{3})?$", printables_rate=1.,
+        expansion_factor=(3.8, .2))
 
