@@ -24,5 +24,5 @@ ENCMAP = {i: "\033[48;5;%dm \033[0;00m" % c for i, c in zip("0123456789", [232,
 
 
 add_map("resistor", ENCMAP, intype="ord", pattern=r"^(?:condensator|resistor)s?(?:[-_]color(?:[-_]code)?)?$",
-        entropy=3.4, printables_rate=.3333333333333333)
+        entropy=3.4, printables_rate=.3333333333333333, expansion_factor=(56., 2.))
 
