@@ -20,8 +20,9 @@ ENCMAP = {
     'A': "NEVER", 'B': "GONNA", 'C': "GIVE", 'D': "YOU", 'E': "UP", 'F': "Never", 'G': "Gonna", 'H': "LET", 'I': "You",
     'J': "DOWN", 'K': "NEver", 'L': "GOnna", 'M': "TURN", 'N': "AROUND", 'O': "AND", 'P': ["DESERT", "DESSERT"],
     'Q': "YOu", 'R': "NEVer", 'S': "gonna", 'T': "TELL", 'U': "A", 'V': "LIE", 'W': "and", 'X': "HURT", 'Y': "you",
-    'Z': "rick astley", ' ': "+", '.': ".", '\n': "\n",
+    'Z': "rick", ' ': "+", '.': ".", '\n': "\n",
     '0': "0", '1': "1", '2': "2", '3': "3", '4': "4", '5': "5", '6': "6", '7': "7", '8': "8", '9': "9",
+    '': "astley", # silent this token for decoding ("rick astley" causes an issue with the separator " ")
 }
 
 
